"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Jsonix = require('jsonix').Jsonix;
const etp = require("etp");
const common = require("./EnergisticsCommons");
const EtpContentType_1 = require("./EtpContentType");
function dottedVersion(str) {
    var retVal = str[0];
    for (var i = 1; i < str.length; i++) {
        retVal += ".";
        retVal += str[i];
    }
    return retVal;
}
var schemas = {
    "witsml1411": require("witsml1411"),
    "witsml20": require("witsml20"),
    "resqml20": require("resqml20"),
    "prodml122": require("prodml122")
};
class DataObjectFactory {
    constructor(context) {
        this.contentType = null;
        if (context.hasOwnProperty("TYPE_NAME")) {
            this.fromNative(context);
        }
        if (typeof context == "string") {
            this.contentType = new EtpContentType_1.EtpContentType(context);
            if (this.contentType.valid) {
                this._packageName = this.contentType.schemaFamily + this.contentType.version.replace(/[v\.]/ig, "");
                this._needsPlural = this.contentType.hasPlural;
            }
            else {
                this._packageName = context;
            }
        }
    }
    getPackage() {
        var module = schemas[this._packageName];
        return module[this._packageName];
    }
    get packageName() {
        return this._packageName;
    }
    getContext() {
        return new Jsonix.Context([this.getPackage()]);
    }
    deserializeString(xmlString) {
        var context = this.getContext();
        var deserializer = context.createUnmarshaller();
        return deserializer.unmarshalString(xmlString);
    }
    deserializeFile(fileName, callback) {
        var context = this.getContext();
        var deserializer = context.createUnmarshaller();
        deserializer.unmarshalFile(fileName, callback);
    }
    serializeString(object) {
        var ct = this.contentType;
        var context = this.getContext();
        var ser = context.createMarshaller();
        if (this._needsPlural) {
            var plural = {
                name: {
                    localPart: ct.pluralName,
                    prefix: context.modules[0].name[0],
                    namespaceURI: context.modules[0].targetNamespace
                },
                value: {
                    version: ct.version,
                }
            };
            plural.value[ct.singularName] = [object];
            return ser.marshalString(plural);
        }
        else {
        }
    }
    fromNative(native) {
        this.contentType = new EtpContentType_1.EtpContentType(DataObjectFactory.contentTypeFromNative(native));
        this._packageName = this.contentType.schemaFamily + this.contentType.version.replace(/[v\.]/ig, "");
        this._needsPlural = this.contentType.hasPlural;
    }
    static contentTypeFromNative(native) {
        var pattern = native.TYPE_NAME.match(/^(\w+ml)(\d+)\.Obj(\w+)$/i);
        return "application/x-" + pattern[1] + "+xml;version=" + dottedVersion(pattern[2]) + ";type=obj_" + pattern[3].toLowerCase();
    }
    static resourceFromNative(native) {
        var ct = new EtpContentType_1.EtpContentType(DataObjectFactory.contentTypeFromNative(native));
        var retVal = new etp.Energistics.Datatypes.Object.Resource();
        retVal.uri = ct.dataType + "(" + native._id + ")";
        retVal.name = native.name;
        retVal.resourceType = "DataObject";
        retVal.objectNotifiable = true;
        retVal.hasChildren = -1;
        retVal.channelSubscribable = (["well", "wellbore", "channel", "channelSet", "log"].indexOf(ct.dataType) >= 0);
        retVal.contentType = ct.contentType;
        if (native.commonData && native.commonData.dTimLastChanged) {
            retVal.lastChanged = common.etpDate(native.commonData.dTimLastChanged.date);
        }
        return retVal;
    }
    static dataObjectFromNative(native) {
        var retVal = {};
        retVal.resource = DataObjectFactory.resourceFromNative(native);
        retVal.contentEncoding = "text/xml";
        var xmlFactory = new DataObjectFactory(retVal.resource.contentType);
        retVal.data = common.String2Buffer(xmlFactory.serializeString(native));
        return retVal;
    }
}
exports.DataObjectFactory = DataObjectFactory;
//# sourceMappingURL=DataObjectFactory.js.map