"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const etp_1 = require("etp");
const BaseHandler_1 = require("../common/BaseHandler");
var ChannelStreaming = etp_1.Energistics.Protocol.ChannelStreaming;
var PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
class ChannelStreamingConsumer extends BaseHandler_1.BaseHandler {
    constructor(sessionManager) {
        super(sessionManager);
        this.sessionManager = sessionManager;
        this._role = "consumer";
        this._protocol = 1;
    }
    start() {
        super.start();
        this.sessionManager.log("Starting new ChannelStreamingConsumer.");
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, 0, 0);
        var message = { role: "ChannelStreamingConsumer", maxMessageRate: 0, maxDataItems: 10 };
        this.sessionManager.send(header, message);
    }
    describe(uris) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelDescribe, 0);
        this.sessionManager.send(header, { uris: uris });
    }
    handleMessage(messageHeader, messageBody) {
        switch (messageHeader.messageType) {
            case ChannelStreaming.MsgChannelMetadata:
                this.emit('metadata', { header: messageHeader, body: messageBody });
                break;
            case ChannelStreaming.MsgChannelData:
                this.emit('data', { header: messageHeader, body: messageBody });
                break;
            default:
                super.handleMessage(messageHeader, messageBody);
                break;
        }
    }
    channelRangeRequest(channelRanges) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelRangeRequest, 0);
        return this.sessionManager.send(header, { channelRanges: channelRanges });
    }
    channelStreamingStart(channelIds) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelStreamingStart, 0);
        return this.sessionManager.send(header, { channels: channelIds });
    }
    channelStreamingStop(channelIds) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelStreamingStop, 0);
        return this.sessionManager.send(header, { channels: channelIds });
    }
}
exports.ChannelStreamingConsumer = ChannelStreamingConsumer;
//# sourceMappingURL=ChannelStreamingConsumer.js.map