"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const etp_1 = require("etp");
const BaseHandler_1 = require("../common/BaseHandler");
var Discovery = etp_1.Energistics.Protocol.Discovery;
var PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
function resource_callback(err, data) {
    if (err) {
        this.handler.log(err);
    }
    else if (data[0]) {
        var resources;
        if (data[0].logCurveInfo) {
            resources = data[0].logCurveInfo.map(function (x) { return { name: x.mnemonic, uri: this._id + "/obj_channel(" + x.mnemonic + ")" }; }, data[0]);
        }
        else {
            resources = data;
        }
        try {
            resources.forEach(function (resource, i, src) {
                console.log(i + " of " + src.length + " messages");
                resource.uri = resource.uri.replace("____", resource.uid);
                var header = this.handler.sessionManager.createHeader(PROTOCOL.Discovery, Discovery.MsgGetResourcesResponse, this.header.messageId);
                if (src.length == 1)
                    header.messageFlags = 0;
                else if (i == src.length - 1)
                    header.messageFlags = 3;
                else
                    header.messageFlags = 1;
                console.log(header.messageFlags);
                this.handler.sessionManager.send(header, { resource: resource });
            }.bind(this));
        }
        catch (e) {
            this.handler.log(e);
        }
    }
}
class DiscoveryStore extends BaseHandler_1.BaseHandler {
    constructor(sessionManager, store) {
        super(sessionManager);
        this.sessionManager = sessionManager;
        this.store = null;
        this.store = store;
        this._role = "store";
        this._protocol = 3;
    }
    handleMessage(messageHeader, messageBody) {
        if (messageHeader.protocol == PROTOCOL.Discovery) {
            switch (messageHeader.messageType) {
                case Discovery.MsgGetResources:
                    messageBody.uri = decodeURI(messageBody.uri);
                    this.log("Discovery: Get " + messageBody.uri);
                    if (this.store)
                        this.store.enum(messageBody.uri, resource_callback.bind({ handler: this, header: messageHeader, body: messageBody }));
                    break;
                default:
                    super.handleMessage(messageHeader, messageBody);
            }
        }
        else {
            throw ("Unsupported protocol {" + messageHeader.protocol + "} in DiscoveryStore");
        }
    }
}
exports.DiscoveryStore = DiscoveryStore;
//# sourceMappingURL=DiscoveryStore.js.map