"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const etp_1 = require("etp");
const BaseHandler_1 = require("../common/BaseHandler");
const EnergisticsCommons_1 = require("../common/EnergisticsCommons");
var StoreNotification = etp_1.Energistics.Protocol.StoreNotification;
var PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
function changeMatchesContext(change, context) {
    return (context.body.request.uri == "/")
        && ((context.body.request.objectTypes.indexOf(change.dataObject.resource.contentType) >= 0)
            || context.body.request.objectTypes.length == 0);
}
function sendChangeIfMatches(request) {
    if (changeMatchesContext(this.change, request)) {
        this.handler.log("Sending " + this.change.changeType + "notification");
        var header, body;
        this.change.dataObject.data = EnergisticsCommons_1.String2Buffer("");
        if (this.change.changeType == "Delete") {
            header = this.handler.sessionManager.createHeader(PROTOCOL.StoreNotification, StoreNotification.MsgDeleteNotification, request.header.messageId);
            body = { delete: this.change };
        }
        else {
            header = this.handler.sessionManager.createHeader(PROTOCOL.StoreNotification, StoreNotification.MsgChangeNotification, request.header.messageId);
            body = { change: this.change };
        }
        this.handler.sessionManager.send(header, body);
    }
}
class StoreNotificationStore extends BaseHandler_1.BaseHandler {
    constructor(sessionManager, store) {
        super(sessionManager);
        this.sessionManager = sessionManager;
        this.store = store;
        this.requests = [];
        this.store = store;
        this._role = "store";
        this._protocol = PROTOCOL.StoreNotification;
    }
    handleMessage(messageHeader, messageBody) {
        if (messageHeader.protocol == PROTOCOL.StoreNotification) {
            switch (messageHeader.messageType) {
                case StoreNotification.MsgNotificationRequest:
                    this.log("Received notification subscription: " + JSON.stringify(messageBody));
                    this.requests.push({ header: messageHeader, body: messageBody });
                    this.store.notificationRequest({ header: messageHeader, body: messageBody }, this.onChangeNotification.bind(this));
                    break;
                default:
                    super.handleMessage(messageHeader, messageBody);
            }
        }
        else {
            throw ("Unsupported protocol {" + messageHeader.protocol + "} in StoreNotification");
        }
    }
    onChangeNotification(change) {
        this.requests.forEach(sendChangeIfMatches, { handler: this, change: change });
    }
}
exports.StoreNotificationStore = StoreNotificationStore;
//# sourceMappingURL=StoreNotificationStore.js.map