"use strict";
const events = require("events");
const Energistics = require("../Energistics");
const witsml = require("../common/WitsmlHelper");
const fs = require("fs");
class LogPlayer extends events.EventEmitter {
    constructor(_config) {
        super();
        this._config = _config;
        this.log = null;
        this.channelCount = 0;
        this.channelMetadata = [];
        this.running = false;
        this.currentRow = 0;
        this.totalRows = 0;
        _config.fileName || (_config.fileName = "");
        var xml = new Energistics.DataObjectFactory("application/x-witsml+xml;version=1.4.1.1;type=obj_log");
        var str = fs.readFileSync(_config.fileName, "utf-8");
        var doc = xml.deserializeString(str);
        this.log = new witsml.LogLoader(null, doc.value.log[0], this);
        this.log.pivot = false;
        this.log.loadLog();
        this.totalRows = this.log.frameData.length;
        for (var i = 0; i < this.log.channelMetadata.length; i++) {
            this.log.channelMetadata[i].status = "Active";
        }
    }
    send() {
        if (this.running && (!this.log.pivot)) {
            var row = this.log.frameData[this.currentRow];
            if (row) {
                var indexMetadata = this.log.channelMetadata[0].indexes[0];
                var t0 = Energistics.decodeIndexValue(indexMetadata, row[0].indexes[0]);
                row[0].indexes[0] = Energistics.encodeIndexValue(indexMetadata, new Date().valueOf());
                this.emit("data", { data: row });
                if (++this.currentRow < this.totalRows) {
                    var nextRow = this.log.frameData[this.currentRow];
                    var t1 = Energistics.decodeIndexValue(indexMetadata, nextRow[0].indexes[0]);
                    if (this._config.speed <= 0) {
                        this.send();
                    }
                    else {
                        var delay = this._config.speed;
                        setTimeout(this.send.bind(this), (t1 - t0) / this._config.speed);
                    }
                }
                else {
                    this.stop();
                }
            }
        }
    }
    channelStreamingStart() {
        this.running = true;
        this.send();
    }
    stop() {
        this.running = false;
    }
    describe(msg) {
        if (msg.body.uris[0] == "/") {
            if (this.log && this.log.logCurveInfo) {
                this.emit('metadata', { channels: this.log.channelMetadata }, msg.header.messageId);
            }
        }
        else {
        }
    }
}
LogPlayer.defaultConfig = {
    fileName: './test/data/log1411.xml',
    speed: 1,
    defaultUri: '/'
};
module.exports = LogPlayer;
//# sourceMappingURL=LogPlayer.js.map