"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const etp = require("../lib/Energistics");
var socket = require('websocket').w3cwebsocket;
var stats = {
    discoveryUris: 0
};
var path = require('path'), config = require('optimist')
    .usage("\nETP Server Ping Utility")
    .default({
    traceMessages: false,
    traceDirectory: 'trace',
    configDirectory: '../config',
    recorderFile: 'recorders.json',
    walkDiscoveryTree: true,
    getDataObjects: false,
    rootUri: "/",
    name: '',
    consumer: true
}).argv;
if (config.help) {
    require('optimist').showHelp();
    process.exit(0);
}
let _connected = false;
let _resources = {};
let _etp = new etp.ETPClient(config);
let _streaming = new etp.ChannelStreamingConsumer(_etp);
let _discovery = new etp.DiscoveryCustomer(_etp);
let _store = new etp.StoreCustomer(_etp);
const servers = require(config.configDirectory + "/" + config.recorderFile);
let server = servers[0];
if (config.name != '') {
    for (let i = 0; i < servers.length; i++) {
        if (config.name == servers[i].name)
            server = servers[i];
    }
}
function bin2string(array) {
    var result = "";
    for (var i = 0; i < array.length; i++) {
        result += String.fromCharCode(array[i]);
    }
    return result;
}
function onSocketConnect(socket) {
    _connected = true;
    console.log('Socket Status: ' + socket.readyState);
    console.log("Requesting session with {" + server.url + "}");
    var thisVersion = { major: 1, minor: 0, revision: 0, patch: 0 };
    _etp.requestSession([
        { protocol: 1, protocolVersion: thisVersion, role: "producer", protocolCapabilities: {} },
        { protocol: 3, protocolVersion: thisVersion, role: "store", protocolCapabilities: {} },
        { protocol: 4, protocolVersion: thisVersion, role: "store", protocolCapabilities: {} }
    ], "etp-ping");
}
function onSocketDisconnect(socket) {
    console.log("Disconnected");
    _connected = false;
}
function onOpenSession(header, message) {
    if (config.walkDiscoveryTree) {
        _discovery.get(config.rootUri);
    }
    if (config.consumer) {
        _streaming.start();
    }
}
function onDiscoverResource(header, message) {
    if (!_resources[message.resource.uri]) {
        console.log(message.resource.uri);
        _resources[message.resource.uri] = message;
        if (message.resource.hasChildren != 0)
            _discovery.get(message.resource.uri);
        if (config.getDataObjects) {
            _store.get(message.resource.uri);
        }
    }
}
function onDataObject(header, message) {
    var uri = new etp.EtpUri(message.dataObject.resource.uri);
    var xml = bin2string(message.dataObject.data);
    console.log(xml);
}
console.log(config);
console.log(server);
_etp.on('log', console.log);
_etp.on('connect', onSocketConnect);
_etp.on('disconnect', onSocketDisconnect);
_etp.on('open', onOpenSession);
if (config.consumer) {
    _etp.registerHandler(1, _streaming);
    _streaming.on('metadata', function (message) {
        console.dir(message.body);
    });
    _streaming.on('data', function (message) {
        console.dir("Received " + message.body.data.length + " points at " + new Date());
    });
}
_etp.registerHandler(3, _discovery);
_discovery.on('resource', onDiscoverResource);
_etp.registerHandler(4, _store);
_store.on('object', onDataObject);
_etp.connect(server, socket);
//# sourceMappingURL=ping.js.map