"use strict";
const etp = require("./Energistics");
const express = require("express");
const body_parser_1 = require("body-parser");
const optimist = require("optimist");
const jwt = require("jsonwebtoken");
const fs = require("fs");
const http = require("http");
const https = require("https");
const ws = require("websocket");
class EtpServer {
    constructor() {
        this.app = express();
        this.recorders = [];
        this.connections = [];
        this.store = null;
        this.app.get('/', function (req, res) {
            res.send('Hello World!');
        });
        this.app.post('/api/*', (req, res, next) => {
            console.log('got api request');
            body_parser_1.json()(req, res, next);
        });
        this.app.use(express.static('public'));
        this.app.use(function (req, res, next) {
            console.log(new Date(), "Express:", req.method, req.url);
            next();
        });
    }
    use(pattern, handler) {
        this.app.use(pattern, handler);
        return null;
    }
    authenticate(request) {
        switch (this.config.autentication) {
            case 'jwt':
                break;
            case 'basic':
                break;
        }
        if (this.config.authentication == "jwt") {
            try {
                var token = request.httpRequest.headers['authorization'];
                var decoded = jwt.verify(token.substr(7), this.config.jwt.signingSecret, { alg: ['HS256'] });
                console.dir(decoded);
                return true;
            }
            catch (e) {
                console.dir(e);
                return false;
            }
        }
        else if (this.config.authentication == "basic") {
        }
        else {
            return true;
        }
        return false;
    }
    createConnection(request) {
        let ralf = new etp.ServerSession(this.config, this.store);
        let realTimeDataSource = new this.providerClass(this.config, this.store);
        realTimeDataSource.store = this.store;
        let producer = ralf.registerHandler(1, new etp.ChannelStreamingProducer(ralf, realTimeDataSource));
        producer.simpleProducer = this.config.simpleProducer;
        if (!this.config.simpleProducer) {
            ralf.registerHandler(3, new etp.DiscoveryStore(ralf, this.store));
            ralf.registerHandler(4, new etp.StoreStore(ralf, this.store));
            ralf.registerHandler(5, new etp.StoreNotificationStore(ralf, this.store));
        }
        ralf.on('log', function (msg) { console.log(msg); });
        ralf.binary = request.httpRequest.headers['etp-encoding'] === 'binary';
        console.log((new Date()) + ': Connection accepted.');
        console.log(request.httpRequest.url);
        ralf.connection = request.accept(etp.WS_PROTOCOL, request.origin);
        ralf.start();
        this.connections[ralf.sessionId] = ralf;
    }
    createServer() {
        if (this.config.useTls) {
            var sslConfig;
            if (this.config.pfxFile != '') {
                sslConfig = {
                    pfx: fs.readFileSync(this.config.pfxFile),
                    passphrase: this.config.passphrase
                };
            }
            else {
                sslConfig = {
                    key: fs.readFileSync(this.config.tlsKey),
                    cert: fs.readFileSync(this.config.tlsCert),
                    ca: fs.readFileSync(this.config.tlsCa),
                    requestCert: this.config.tlsRequestCert,
                    rejectUnauthorized: this.config.tlsRejectUnauthorized
                };
            }
            this.server = https.createServer(sslConfig, this.app);
        }
        else {
            this.server = http.createServer(this.app);
        }
    }
    startRecorders() {
        this.recorders = require(this.config.configDirectory + "/" + this.config.recorderFile);
        for (var i = 0; i < this.recorders.length; i++) {
            var recorder = this.recorders[i];
            if (recorder.active == true) {
                recorder.store = this.store;
                new this.consumerClass(recorder).start();
            }
        }
    }
    onStoreConnect() {
        if (this.config.startRecorders && this.store) {
            console.log("Starting recorders.");
            this.startRecorders();
        }
    }
    onUpgradeRequest(request) {
        console.log("Upgrade Request: ");
        console.dir(request.httpRequest.headers);
        for (var queryParameter in request.resourceURL.query) {
            request.httpRequest.headers[queryParameter.toLowerCase()] = request.resourceURL.query[queryParameter];
        }
        if (this.authenticate(request)) {
            let sessionId = request.httpRequest.headers['etp-sessionid'];
            if (sessionId != null) {
                let session = this.connections[sessionId];
                if (session) {
                    session.connection = request.accept(etp.WS_PROTOCOL, request.origin);
                    session.resume();
                }
                else {
                    request.reject(410);
                }
            }
            else {
                this.createConnection(request);
            }
        }
        else {
            request.reject(401, "Authentication failed.");
        }
    }
    run() {
        this.configure();
        this.createServer();
        this.server.listen(this.config.wsPort, function () {
            console.log((new Date()) + ' Server is listening on port ' + this.wsPort);
        }.bind(this.config));
        let socketServer = new ws.server({ httpServer: this.server });
        socketServer.on('request', this.onUpgradeRequest.bind(this));
    }
    configure() {
        var argv = optimist
            .usage("\nETP Server for Node.js")
            .default({
            exitOnClose: false,
            wsPort: 8081,
            simpleStreamer: false,
            traceMessages: false,
            traceDirectory: 'trace',
            databaseConnectString: 'mongodb://localhost:27017/test',
            storeModule: '../lib/providers/MongoStore',
            providerModule: '../lib/providers/StoreProvider',
            consumerModule: '../lib/providers/RecordingConsumer',
            configDirectory: '../../config',
            recorderFile: 'recorders.json',
            authentication: 'none',
            startRecorders: false,
            pfxFile: '',
            passphrase: '',
            useTls: false,
            tlsKey: '',
            tlsCert: '',
            tlsCa: '',
            tlsRequestCert: false,
            tlsRejectUnauthorized: false,
            jwt: {
                signingSecret: process.env.JWT_SIGNING_SECRET
            }
        }).argv;
        if (argv.storeModule != "null") {
            this.storeClass = require(argv.storeModule);
        }
        this.providerClass = require(argv.providerModule);
        optimist.default(this.providerClass.defaultConfig);
        this.consumerClass = require(argv.consumerModule);
        this.config = optimist.parse(process.argv);
        if (this.storeClass != null) {
            this.store = new this.storeClass(argv);
            this.store.on("connect", this.onStoreConnect.bind(this));
        }
        if (this.config.help) {
            optimist.showHelp();
            process.exit(0);
        }
        console.log(this.config);
    }
}
module.exports = EtpServer;
//# sourceMappingURL=EtpServer.js.map