"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events = require("events");
const etp_1 = require("etp");
const EnergisticsCommons_1 = require("./EnergisticsCommons");
var Core = etp_1.Energistics.Protocol.Core;
var PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
class ETPCore extends events.EventEmitter {
    constructor(configuration, store = null) {
        super();
        this.connection = null;
        this.name = "Energistics";
        this.config = {};
        this.messageId = 0;
        this.schemaCache = new etp_1.SchemaCache();
        this.sessionId = EnergisticsCommons_1.NO_SESSION;
        this.binary = true;
        this.handlers = [];
        this.store = null;
        this.stats = {
            blocks: 0,
            rows: 0,
            dataPoints: 0,
            bytesSent: 0,
            bytesReceived: 0,
            messagesSent: 0,
            messagesReceived: 0,
            connectDate: new Date(),
            firstDataBlock: null,
            lastDataBlock: null
        };
        this.config = configuration;
    }
    acknowledge(messageId) {
        var header = this.createHeader(PROTOCOL.Core, Core.MsgAcknowledge, messageId, 0);
        var message = {};
        this.send(header, message);
    }
    closeSession() {
        if (this.sessionId) {
            this.send(this.createHeader(PROTOCOL.Core, Core.MsgCloseSession, 0, 0), { sessionId: this.sessionId });
            this.sessionId = EnergisticsCommons_1.NO_SESSION;
        }
    }
    createHeader(protocol, messageType, correlationId = 0, messageFlags = 0) {
        return {
            protocol: protocol,
            messageType: messageType,
            messageId: ++this.messageId,
            correlationId: correlationId,
            messageFlags: messageFlags
        };
    }
    isInSession() {
        return this.connection != null
            && this.connection.readyState == 1
            && this.sessionId != EnergisticsCommons_1.NO_SESSION;
    }
    handleMessage(messageHeader, messageBody) {
        switch (messageHeader.messageType) {
            case Core.MsgAcknowledge:
                this.onAcknowledge(messageHeader, messageBody);
                break;
            case Core.MsgProtocolException:
                this.onException(messageHeader, messageBody);
                break;
        }
    }
    log(message) {
        this.emit('log', message);
    }
    onCloseSession(header, message) {
        var i;
        for (i = 0; i < this.handlers.length; i++) {
            if (this.handlers[i])
                this.handlers[i].stop();
        }
        this.log("Received CloseSession message for session {" + this.sessionId + "} Closing WebSocket.");
        this.sessionId = EnergisticsCommons_1.NO_SESSION;
        this.connection.close();
        this.emit('close');
    }
    onAcknowledge(header, message) {
        this.emit('acknowledge', header, message);
    }
    onException(header, message) {
        this.log("EXCEPTION: " + message.errorMessage);
        this.emit("exception", header, message);
    }
    registerHandler(id, handler) {
        this.handlers[id] = handler;
        return handler;
    }
    send(header, message) {
    }
    sendException(errorCode, errorMessage, correlationId) {
        var header = this.createHeader(PROTOCOL.Core, Core.MsgProtocolException, correlationId, 0);
        var message = { errorCode: errorCode, errorMessage: errorMessage };
        this.send(header, message);
    }
}
exports.ETPCore = ETPCore;
//# sourceMappingURL=ETPCore.js.map