"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const etp_1 = require("etp");
const BaseHandler_1 = require("../common/BaseHandler");
var ChannelStreaming = etp_1.Energistics.Protocol.ChannelStreaming;
var PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
class ChannelStreamingProducer extends BaseHandler_1.BaseHandler {
    constructor(sessionManager, dataSource) {
        super(sessionManager);
        this.sessionManager = sessionManager;
        this.dataSource = dataSource;
        this.nextChannelId = 0;
        this.simpleStreamer = false;
        this._role = "producer";
        this._protocol = 1;
        this.simpleStreamer = sessionManager.config.simpleStreamer;
    }
    start() {
        this.dataSource.on('metadata', this.onMetadata.bind(this));
        this.dataSource.on('data', this.onData.bind(this));
        if (this.simpleStreamer) {
            this.dataSource.describe({ header: { correlationId: 0 }, body: { uris: ["/"] } });
            this.dataSource.channelStreamingStart([]);
        }
    }
    stop() {
        if (this.dataSource)
            this.dataSource.stop();
        this.sessionManager = null;
    }
    handleMessage(messageHeader, messageBody) {
        switch (messageHeader.messageType) {
            case 0:
                this.start();
                break;
            case ChannelStreaming.MsgChannelDescribe:
                try {
                    this.dataSource.describe({ header: messageHeader, body: messageBody });
                }
                catch (exception) {
                    this.sessionManager.log(JSON.stringify(exception));
                    this.sessionManager.sendException(0, "Unable to describe URI " + messageBody.uris, messageHeader.messageId);
                }
                break;
            case ChannelStreaming.MsgChannelRangeRequest:
                this.dataSource.rangeRequest({ header: messageHeader, body: messageBody });
                break;
            case ChannelStreaming.MsgChannelStreamingStart:
                this.dataSource.startStreaming(messageBody.channels);
                break;
            case ChannelStreaming.MsgChannelStreamingStop:
                this.dataSource.stopStreaming(messageBody.channels);
                break;
            default:
                super.handleMessage(messageHeader, messageBody);
                break;
        }
    }
    onData(channelData, correlationId) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelData, correlationId);
        this.sessionManager.send(header, channelData);
    }
    onMetadata(channelMetadata, correlationId) {
        var header = this.sessionManager.createHeader(PROTOCOL.ChannelStreaming, ChannelStreaming.MsgChannelMetadata, correlationId);
        this.sessionManager.send(header, channelMetadata);
    }
    newChannelId() {
        return this.nextChannelId++;
    }
}
exports.ChannelStreamingProducer = ChannelStreamingProducer;
//# sourceMappingURL=ChannelStreamingProducer.js.map