"use strict";
const events = require("events");
class StoreProvider extends events.EventEmitter {
    constructor(config, store) {
        super();
        this.config = config;
        this.store = store;
        this.channelMetadata = {};
        this.activeChannels = {};
        this.running = true;
        store.on("channel-point", this.onChannelData.bind(this));
        store.on("channel-points", this.onChannelDataBlock.bind(this));
    }
    rangeRequest(msg) {
        var message = msg.body;
        for (var i = 0; i < message.channelRanges.length; i++) {
            var channelRange = message.channelRanges[i];
            for (var j = 0; j < channelRange.channelId.length; j++) {
                var channelId = channelRange.channelId[j];
                var channelInfo = this.channelMetadata[channelId];
                if (channelInfo) {
                    this.store.getChannelRange(channelInfo, channelRange.startIndex, channelRange.endIndex, function (err, data) {
                        if (!err) {
                            this.provider.emit('data', { data: data }, this.message.header.messageId);
                        }
                    }.bind({ provider: this, message: msg }));
                }
            }
        }
    }
    startStreaming(channels) {
        for (var i = 0; i < channels.length; i++) {
            var csi = channels[i];
            this.activeChannels[csi.channelId] = csi;
        }
    }
    stop() {
        if (this.running) {
            this.running = false;
            this.activeChannels = [];
            this.store.removeListener("channel-point", this.onChannelData.bind(this));
        }
    }
    stopStreaming(channels) {
        for (var i = 0; i < channels.length; i++) {
            delete this.activeChannels[channels[i]];
        }
    }
    handleObject(err, data) {
    }
    describe(msg) {
        this.store.describeChannels(msg.body.uris, function (err, data) {
            for (var i = 0; i < data.length; i++) {
                var channelMetadata = data[i];
                this.provider.channelMetadata[channelMetadata.channelId] = channelMetadata;
                this.provider.emit('metadata', { channels: [channelMetadata] }, this.request.header.messageId);
            }
        }.bind({ provider: this, request: msg }));
    }
    onChannelData(err, point) {
        if (this.activeChannels[point.channelId]) {
            this.emit('data', { data: [point] });
        }
    }
    onChannelDataBlock(err, data) {
        if (err) {
        }
        else if ((data != null) && (data.length > 0) && (this.running == true)) {
            var myData = data.filter(function (point) {
                return this.activeChannels[point.channelId] != null;
            }.bind(this));
            if (myData.length > 0) {
                myData[0].indexes[0] = data[0].indexes[0];
                this.emit("data", { data: myData });
            }
        }
    }
}
module.exports = StoreProvider;
//# sourceMappingURL=StoreProvider.js.map